/*
 *  Copyright (C) 2007 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <zlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include "SDL.h"



#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "psp_global.h"
#include "psp_sdl.h"

  GCHESS_t GCHESS;

int
psp_global_reset()
{
  psp_chess_cursor_reset();
  psp_play_reset();

  psp_chess_change_color(GCHESS.chess_player_color);
  psp_chess_change_search_depth(GCHESS.chess_search_depth);
  psp_chess_change_book(GCHESS.chess_book);
# if 0 //LUDO: TO_BE_DONE !
  psp_chess_change_limit(GCHESS.chess_time_limit, GCHESS.chess_move_limit);
# endif

  return 0;
}

int
psp_global_init()
{
  GCHESS.chess_level         = 0;
  GCHESS.chess_search_depth  = 0;
  GCHESS.chess_book          = BOOKRAND;
  GCHESS.chess_game_mode     = GCHESS_HUMAN_VS_COMPUTER;
# if 0 //LUDO: TO_BE_DONE !
  GCHESS.chess_time_limit    = 0;
  GCHESS.chess_move_limit    = 0;
# endif
  GCHESS.chess_player_color  = white;
  GCHESS.chess_side_color    = white;
  GCHESS.chess_show_coord    = 1;
  GCHESS.chess_last_move     = 1;
  GCHESS.psp_cpu_clock       = 200;
  GCHESS.psp_screenshot_mode = 0;

  GCHESS.black_color = psp_sdl_rgb( 0x90, 0x90, 0x90);
  GCHESS.white_color = psp_sdl_rgb( 0xe0, 0xe0, 0xe0);
  GCHESS.select_color = psp_sdl_rgb(0xff, 0xff, 0x0);
  GCHESS.valid_color  = psp_sdl_rgb(0xd0, 0xd0, 0x0);
  GCHESS.last_color   = psp_sdl_rgb(0x50, 0xa0, 0xd0);

  strcpy(GCHESS.game_filename, "new_game.pgn");

  psp_parse_configuration();
 
  gp2xPowerSetClockFrequency(GCHESS.psp_cpu_clock, GCHESS.psp_cpu_clock, GCHESS.psp_cpu_clock/2);
  psp_global_reset();

  return 0;
}

int
psp_save_configuration(void)
{
  char  FileName[MAX_PATH];
  FILE* FileDesc;
  int   error = 0;

  strcpy(FileName, GCHESS.psp_homedir);
  strcat(FileName, "/gp2xgchess.cfg");

  FileDesc = fopen(FileName, "w");
  if (FileDesc != (FILE *)0 ) {

    fprintf(FileDesc, "psp_cpu_clock=%d\n"     , GCHESS.psp_cpu_clock);
    fprintf(FileDesc, "chess_game_mode=%d\n"   , GCHESS.chess_game_mode);
    fprintf(FileDesc, "chess_level=%d\n"       , GCHESS.chess_level);
    fprintf(FileDesc, "chess_book=%d\n"        , GCHESS.chess_book);
    fprintf(FileDesc, "chess_search_depth=%d\n", GCHESS.chess_search_depth);
    fprintf(FileDesc, "chess_player_color=%d\n", GCHESS.chess_player_color);
    fprintf(FileDesc, "chess_side_color=%d\n"  , GCHESS.chess_side_color);
    fprintf(FileDesc, "chess_show_coord=%d\n"  , GCHESS.chess_show_coord);
    fprintf(FileDesc, "chess_last_move=%d\n"   , GCHESS.chess_last_move);
    fprintf(FileDesc, "white_color=0x%x\n"       , GCHESS.white_color);
    fprintf(FileDesc, "black_color=0x%x\n"       , GCHESS.black_color);
    fprintf(FileDesc, "select_color=0x%x\n"      , GCHESS.select_color);
    fprintf(FileDesc, "valid_color=0x%x\n"       , GCHESS.valid_color);
    fprintf(FileDesc, "last_color=0x%x\n"        , GCHESS.last_color);

    fclose(FileDesc);

  } else {
    error = 1;
  }

  return error;
}

int
psp_parse_configuration(void)
{
  char  Buffer[512];
  char  FileName[MAX_PATH+1];
  char *Scan;
  FILE* FileDesc;
  int   Value;

  sprintf(FileName, GCHESS.psp_homedir);
  strcat(FileName, "/gp2xgchess.cfg");

  FileDesc = fopen(FileName, "r");
  if (FileDesc == (FILE *)0 ) return 0;

  while (fgets(Buffer,512, FileDesc) != (char *)0) {

    Scan = strchr(Buffer,'\n');
    if (Scan) *Scan = '\0';
    /* For this #@$% of windows ! */
    Scan = strchr(Buffer,'\r');
    if (Scan) *Scan = '\0';
    if (Buffer[0] == '#') continue;

    Scan = strchr(Buffer,'=');
    if (! Scan) continue;

    *Scan = '\0';
    Scan++;
    if ((Scan[0] == '0') && (Scan[1] == 'x')) {
      sscanf(Scan, "%x", &Value);
    } else {
      Value = atoi(Scan);
    }

    if (!strcasecmp(Buffer,"psp_cpu_clock")) GCHESS.psp_cpu_clock = Value;
    else
    if (!strcasecmp(Buffer,"chess_level")) GCHESS.chess_level = Value;
    else
    if (!strcasecmp(Buffer,"chess_search_depth")) GCHESS.chess_search_depth = Value;
    else
    if (!strcasecmp(Buffer,"chess_book")) GCHESS.chess_book = Value;
    else
    if (!strcasecmp(Buffer,"chess_game_mode")) GCHESS.chess_game_mode = Value;
    else
    if (!strcasecmp(Buffer,"chess_player_color")) GCHESS.chess_player_color = Value;
    else
    if (!strcasecmp(Buffer,"chess_side_color")) GCHESS.chess_side_color = Value;
    else
    if (!strcasecmp(Buffer,"chess_show_coord")) GCHESS.chess_show_coord = Value;
    else
    if (!strcasecmp(Buffer,"chess_last_move")) GCHESS.chess_last_move = Value;
    else
    if (!strcasecmp(Buffer,"white_color")) GCHESS.white_color = Value;
    else
    if (!strcasecmp(Buffer,"black_color")) GCHESS.black_color = Value;
    else
    if (!strcasecmp(Buffer,"select_color")) GCHESS.select_color = Value;
    else
    if (!strcasecmp(Buffer,"valid_color")) GCHESS.valid_color = Value;
    else
    if (!strcasecmp(Buffer,"last_color")) GCHESS.last_color = Value;
  }

  fclose(FileDesc);

  return 0;
}


