/*
 *  Copyright (C) 2007 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

# ifndef _GP2X_GLOBAL_H_
# define _GP2X_GLOBAL_H_

#include "engine/common.h"

#include <time.h>

# ifndef CLK_TCK
# define CLK_TCK  CLOCKS_PER_SEC
# endif

#define CPU_CLOCK_IDLE 60
#define CPU_CLOCK_STD 200

# define MAX_PATH   128

# define GCHESS_HUMAN_VS_COMPUTER     0
# define GCHESS_HUMAN_VS_HUMAN        1
# define GCHESS_COMPUTER_VS_COMPUTER  2

# define GCHESS_MAX_DEPTH            10
# define GCHESS_MAX_TIME            120
# define GCHESS_MAX_MOVE            100

# define GCHESS_BOARD_WIDTH           8

  typedef struct GCHESS_t {

    char     psp_homedir[MAX_PATH];
    char     game_filename[MAX_PATH];
             
    int      move_x[2];
    int      move_y[2];
    int      move_step;

    int      black_color;
    int      white_color;
    int      select_color;
    int      valid_color;
    int      last_color;

    int      chess_player_color; /* white or black ? */
    int      chess_side_color;   /* white or black ? */

    int      chess_level;
    int      chess_book;
    int      chess_search_depth;
# if 0 //LUDO: TO_BE_DONE !
    int      chess_move_limit;
    int      chess_time_limit;
# endif
    int      chess_game_mode; /* vs computer ? */
    int      chess_show_coord;
    int      chess_last_move;
    
    int      psp_cpu_clock;
    int      psp_screenshot_mode;
    int      psp_screenshot_id;

  } GCHESS_t;

  extern GCHESS_t GCHESS;

  extern int psp_global_reset();
  extern int psp_parse_configuration(void);
  extern int psp_save_configuration(void);

# endif
