/*
 *  Copyright (C) 2007 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_ttf.h>
#include "gp2x_psp.h"

#include "psp_global.h"
#include "psp_danzeff.h"
#include "psp_sdl.h"
#include "psp_menu.h"
#include "psp_fmgr.h"
#include "psp_menu_help.h"
#include "psp_kbd.h"
#include "psp_chess.h"
#include "psp_danzeff.h"

# define MENU_FILENAME     0
# define MENU_GAME_MODE    1
# define MENU_PLAYER_COLOR 2
# define MENU_SIDE_COLOR   3
# define MENU_SHOW_COORD   4
# define MENU_LAST_MOVE    5
# if 0
# define MENU_TIME_LIMIT   4
# define MENU_MOVE_LIMIT   5
# endif
# define MENU_DEPTH        6
# define MENU_BOOK         7
# define MENU_CLOCK        8
# define MENU_SCREENSHOT   9

# define MENU_HELP        10

# define MENU_NEW_GAME    11
# define MENU_RESIGN_GAME 12
# define MENU_LOAD_GAME   13
# define MENU_SAVE_GAME   14
# define MENU_BACK        15
# define MENU_SAVE_CFG    16
# define MENU_EXIT        17

# define MAX_MENU_ITEM (MENU_EXIT + 1)

  static menu_item_t menu_list[] =
  { 
    { "Filename         : " },
    { "Game mode        : " },
    { "Player color     : " },
    { "Bottom color     : " },
    { "Show coordinates : " },
    { "Show last move   : " },
# if 0
    { "Time limit       : " },
    { "Move limit       : " },
# endif
    { "Search depth     : " },
    { "Book mode        : " },
    { "Clock frequency  : " },
    { "Save Screenshot  : " },

    { "Help" },

    { "New game" },
    { "Resign game" },
    { "Load game" },
    { "Save game" },
    { "Back to game" },
    { "Save config" },
    { "Exit" },
  };

  static int cur_menu_id = MENU_BACK;

  static int psp_menu_dirty = 1;

  static int chess_player_color = white;
  static int chess_side_color   = white;
  static int chess_game_mode    = GCHESS_HUMAN_VS_COMPUTER;
  static int chess_show_coord   = 0;
  static int chess_last_move    = 1;
  static int chess_search_depth = 0;
  static int chess_book         = BOOKRAND;
# if 0 //LUDO: TO_BE_DONE !
  static int chess_move_limit   = 0;
  static int chess_time_limit   = 0;
# endif
  static int psp_cpu_clock      = 200;

# define FILENAME_FIELD_WIDTH      25

void
string_fill_with_space(char *buffer, int size)
{
  int length = strlen(buffer);
  int index;

  for (index = length; index < size; index++) {
    buffer[index] = ' ';
  }
  buffer[size] = 0;
}

static void 
psp_display_screen_menu(void)
{
  char buffer[128];
  int menu_id = 0;
  int color   = 0;
  int x       = 0;
  int y       = 0;
  int y_step  = 0;

  //if (psp_menu_dirty) 
  {
    psp_sdl_blit_background();
    //psp_menu_dirty = 0;
  }
  
  x      = 10;
  y      = 15;
  y_step = 10;
  
  for (menu_id = 0; menu_id < MAX_MENU_ITEM; menu_id++) {
    color = GP2X_MENU_TEXT_COLOR;
    if (cur_menu_id == menu_id) color = GP2X_MENU_SEL_COLOR;
    else 
    if (menu_id == MENU_EXIT) color = GP2X_MENU_RED_COLOR;
    else
    if (menu_id == MENU_HELP) color = GP2X_MENU_GREEN_COLOR;

    psp_sdl_back2_print(x, y, menu_list[menu_id].title, color);

    if (menu_id == MENU_FILENAME) {
      int length = strlen(GCHESS.game_filename);
      int first = 0;
      if (length > (FILENAME_FIELD_WIDTH-1)) {
        first = length - (FILENAME_FIELD_WIDTH-1);
      }
      sprintf(buffer, "%s", GCHESS.game_filename + first);
      if (menu_id == cur_menu_id) strcat(buffer, "_");
      string_fill_with_space(buffer, FILENAME_FIELD_WIDTH+1);
      psp_sdl_back2_print(130, y, buffer, color);

    } else
    if (menu_id == MENU_PLAYER_COLOR) {
      if (chess_player_color == black) strcpy(buffer, "black");
      else                             strcpy(buffer, "white");
      string_fill_with_space(buffer, 5);
      psp_sdl_back2_print(130, y, buffer, color);
    } else
    if (menu_id == MENU_SIDE_COLOR) {
      if (chess_side_color == black) strcpy(buffer, "black");
      else                           strcpy(buffer, "white");
      string_fill_with_space(buffer, 5);
      psp_sdl_back2_print(130, y, buffer, color);
    } else
    if (menu_id == MENU_DEPTH) {
      if (chess_search_depth) {
        sprintf(buffer, "%d", chess_search_depth);
      } else {
        strcpy(buffer, "auto");
      }
      string_fill_with_space(buffer, 5);
      psp_sdl_back2_print(130, y, buffer, color);
    } else
# if 0 //LUDO: TO_BE_DONE !
    if (menu_id == MENU_TIME_LIMIT) {
      if (chess_time_limit) {
        sprintf(buffer, "%d min", chess_time_limit);
      } else {
        strcpy(buffer, "none");
      }
      string_fill_with_space(buffer, 10);
      psp_sdl_back2_print(130, y, buffer, color);
    } else
    if (menu_id == MENU_MOVE_LIMIT) {
      if (chess_move_limit) {
        sprintf(buffer, "%d", chess_move_limit);
      } else {
        strcpy(buffer, "none");
      }
      string_fill_with_space(buffer, 10);
      psp_sdl_back2_print(130, y, buffer, color);
    } else
# endif
    if (menu_id == MENU_BOOK) {

      if (chess_book == BOOKOFF) strcpy(buffer, "off");
      else
      if (chess_book == BOOKRAND) strcpy(buffer, "random");
      else
      if (chess_book == BOOKBEST) strcpy(buffer, "best");
      else
      if (chess_book == BOOKWORST) strcpy(buffer, "worst");
      else
      /* if (chess_book == BOOKPREFER) */ strcpy(buffer, "prefer");

      string_fill_with_space(buffer, 15);
      psp_sdl_back2_print(130, y, buffer, color);
    } else
    if (menu_id == MENU_GAME_MODE) {
      if (chess_game_mode == GCHESS_HUMAN_VS_COMPUTER) strcpy(buffer, "Human vs GP2X");
      else
      if (chess_game_mode == GCHESS_HUMAN_VS_HUMAN) strcpy(buffer, "Human vs Human");
      else                                    strcpy(buffer, "GP2X vs GP2X");
      string_fill_with_space(buffer, 15);
      psp_sdl_back2_print(130, y, buffer, color);
    } else
    if (menu_id == MENU_SHOW_COORD) {
      if (chess_show_coord) strcpy(buffer, "yes");
      else                  strcpy(buffer, "no");
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(130, y, buffer, color);
    } else
    if (menu_id == MENU_LAST_MOVE) {
      if (chess_last_move) strcpy(buffer, "yes");
      else                 strcpy(buffer, "no");
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(130, y, buffer, color);
    } else
    if (menu_id == MENU_CLOCK) {
      sprintf(buffer,"%d", psp_cpu_clock);
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(130, y, buffer, color);
    } else
    if (menu_id == MENU_SCREENSHOT) {
      sprintf(buffer,"%d", GCHESS.psp_screenshot_id);
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(130, y, buffer, color);
      y += y_step;
    } else
    if (menu_id == MENU_SAVE_CFG) {
      y += y_step;
    } else
    if (menu_id == MENU_HELP) {
      y += y_step;
    } else
    if (menu_id == MENU_EXIT) {
      y += y_step;
    }

    y += y_step;
  }
}

void
psp_main_menu_new_game(void)
{
  psp_chess_new_game();
}

static void
psp_main_menu_validate(void)
{
  int need_to_reset = 0;
  GCHESS.psp_cpu_clock = psp_cpu_clock;
 
  if (chess_player_color != GCHESS.chess_player_color) {
    psp_chess_change_color(chess_player_color);
  }
  if (chess_search_depth != GCHESS.chess_search_depth) {
    psp_chess_change_search_depth(chess_search_depth);
  }
  if (chess_book != GCHESS.chess_book) {
    psp_chess_change_book(chess_book);
  }
# if 0 //LUDO: TO_BE_DONE 
  if ((chess_move_limit != GCHESS.chess_move_limit) ||
      (chess_time_limit != GCHESS.chess_time_limit)) {
    psp_chess_change_limit(chess_time_limit, chess_move_limit);
  }
# endif
  GCHESS.chess_game_mode = chess_game_mode;
  GCHESS.chess_show_coord = chess_show_coord;
  GCHESS.chess_last_move  = chess_last_move;
  GCHESS.chess_side_color = chess_side_color;

  if (need_to_reset) {
    psp_main_menu_new_game();
  }
  gp2xPowerSetClockFrequency(GCHESS.psp_cpu_clock);
}

void
psp_main_menu_resign_game(void)
{
  psp_chess_resign_game();
}

int
psp_main_menu_confirm_overwrite()
{
  gp2xCtrlData c;

  psp_display_screen_menu();
  psp_sdl_back2_print(140, 160, "X overwrite B append A cancel", GP2X_MENU_WARNING_COLOR);
  psp_sdl_flip();
  psp_kbd_wait_no_button();

  do
  {
    gp2xCtrlReadBufferPositive(&c, 1);
    c.Buttons &= GP2X_ALL_BUTTON_MASK;

    if (c.Buttons & GP2X_CTRL_CROSS) {
      return 1;
    } else 
    if (c.Buttons & GP2X_CTRL_CIRCLE) {
      return 2;
    } else 
    if (c.Buttons & GP2X_CTRL_SQUARE) {
      return 0;
    }

  } while (c.Buttons == 0);
  psp_kbd_wait_no_button();

  return 0;
}

static void
psp_main_menu_save_config()
{
  int error;

  psp_main_menu_validate();
  error = psp_save_configuration();

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(140, 160, "File saved !", 
                       GP2X_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(140, 160, "Can't save file !", 
                       GP2X_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
}

static void
psp_main_menu_screenshot(void)
{
  GCHESS.psp_screenshot_mode = 1;
}

int
psp_main_menu_exit(void)
{
  gp2xCtrlData c;

  psp_display_screen_menu();
  psp_sdl_back2_print(140, 160, "press X to confirm !", GP2X_MENU_WARNING_COLOR);
  psp_menu_dirty = 1;
  psp_sdl_flip();

  psp_kbd_wait_no_button();

  do
  {
    gp2xCtrlReadBufferPositive(&c, 1);
    c.Buttons &= GP2X_ALL_BUTTON_MASK;

    if (c.Buttons & GP2X_CTRL_CROSS) psp_sdl_exit(0);

  } while (c.Buttons == 0);

  psp_kbd_wait_no_button();

  return 0;
}

static void
psp_main_menu_init(void)
{
  psp_cpu_clock       = GCHESS.psp_cpu_clock;
  chess_game_mode     = GCHESS.chess_game_mode;
  chess_search_depth  = GCHESS.chess_search_depth;
# if 0 //LUDO: TO_BE_DONE !
  chess_time_limit    = GCHESS.chess_time_limit;
  chess_move_limit    = GCHESS.chess_move_limit;
# endif
  chess_book          = GCHESS.chess_book;
  chess_player_color  = GCHESS.chess_player_color;
  chess_show_coord    = GCHESS.chess_show_coord;
  chess_last_move     = GCHESS.chess_last_move;
}


void
psp_main_menu_player_color( int step )
{
  if (chess_player_color == black) chess_player_color = white;
  else                             chess_player_color = black;
}

void
psp_main_menu_side_color( int step )
{
  if (chess_side_color == black) chess_side_color = white;
  else                           chess_side_color = black;
}

void
psp_main_menu_search_depth( int step )
{
  if (step > 0) {
    if (chess_search_depth <= GCHESS_MAX_DEPTH) chess_search_depth++;
    else                                chess_search_depth = 0;
  } else 
  if (step < 0) {
    if (chess_search_depth > 0) chess_search_depth--;
    else                 chess_search_depth = GCHESS_MAX_DEPTH;
  }
}

# if 0 //LUDO: TO_BE_DONE !
void
psp_main_menu_time_limit( int step )
{
  if (step > 0) {
    if (chess_time_limit <= GCHESS_MAX_TIME) chess_time_limit++;
    else                                chess_time_limit = 0;
  } else 
  if (step < 0) {
    if (chess_time_limit > 0) chess_time_limit--;
    else                 chess_time_limit = GCHESS_MAX_TIME;
  }
}

void
psp_main_menu_move_limit( int step )
{
  if (step > 0) {
    if (chess_move_limit <= GCHESS_MAX_TIME) chess_move_limit++;
    else                                chess_move_limit = 0;
  } else 
  if (step < 0) {
    if (chess_move_limit > 0) chess_move_limit--;
    else                 chess_move_limit = GCHESS_MAX_TIME;
  }
}
# endif

void
psp_main_menu_book( int step )
{
  if (step > 0) {
    if (chess_book < BOOKPREFER) chess_book++;
    else                         chess_book = 0;
  } else 
  if (step < 0) {
    if (chess_book > 0) chess_book--;
    else                chess_book = BOOKPREFER;
  }
}

void
psp_main_menu_game_mode( int step )
{
  if (step > 0) {
    if (chess_game_mode < GCHESS_COMPUTER_VS_COMPUTER) chess_game_mode++;
    else                                               chess_game_mode = 0;
  } else {
    if (chess_game_mode > 0) chess_game_mode--;
    else                  chess_game_mode = GCHESS_COMPUTER_VS_COMPUTER;
  }
}

void
psp_main_menu_show_coord( int step )
{
  chess_show_coord = ! chess_show_coord;
}

void
psp_main_menu_last_move( int step )
{
  chess_last_move = ! chess_last_move;
}

static void
psp_main_menu_clock(int step)
{
  if (step > 0) {
    psp_cpu_clock += 10;
    if (psp_cpu_clock > 260) psp_cpu_clock = 260;
  } else {
    psp_cpu_clock -= 10;
    if (psp_cpu_clock < 60) psp_cpu_clock = 60;
  }
}

void
psp_main_menu_filename_add_char(int a_char)
{
  int length = strlen(GCHESS.game_filename);
  if (length < (MAX_PATH-2)) {
    GCHESS.game_filename[length] = a_char;
    GCHESS.game_filename[length+1] = 0;
  }
}

void
psp_main_menu_filename_clear()
{
  GCHESS.game_filename[0] = 0;
}

void
psp_main_menu_filename_del()
{
  int length = strlen(GCHESS.game_filename);
  if (length >= 1) {
    GCHESS.game_filename[length - 1] = 0;
  }
}


static int
psp_main_menu_save_game()
{
  int error;
  char pgn_filename[MAX_PATH];

  if (GCHESS.game_filename[0] == 0) {
    strcpy(GCHESS.game_filename, "newgame");
  }

  char* scan_char = strrchr(GCHESS.game_filename, '.');
  if (! scan_char) {
    strcat(GCHESS.game_filename, ".pgn");
  }
  strcpy(pgn_filename, GCHESS.psp_homedir);
  strcat(pgn_filename, "/save/");
  strcat(pgn_filename, GCHESS.game_filename);

  int do_write = 0;

  struct stat aStat;
  if (stat(pgn_filename, &aStat)) {
    do_write = 1;
  } else {
    do_write = psp_main_menu_confirm_overwrite();
  }
  if (! do_write) {
    return 0;
  }
  if (do_write == 1) {
    error = psp_chess_save_pgn(pgn_filename);
  } else {
    error = psp_chess_append_pgn(pgn_filename);
  }

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(140, 160, "File saved !", 
                       GP2X_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(140, 160, "Can't save file !", 
                       GP2X_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  return error;
}

int
psp_main_menu_load_game(void)
{
  int ret;

  ret = psp_fmgr_menu_load(GP2X_FMGR_FORMAT_PGN);
  if (ret ==  1) /* load OK */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(140, 160, "File loaded !", 
                       GP2X_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
    return 1;
  }
  else 
  if (ret == -1) /* Load Error */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(140, 160, "Can't load file !", GP2X_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  return 0;
}

int 
psp_main_menu(void)
{
  gp2xCtrlData c;
  long        new_pad;
  long        old_pad;
  int         last_time;
  int         end_menu;

  int         danzeff_mode = 0;
  int         danzeff_key;

  int         step = 0;

  psp_kbd_wait_no_button();

  old_pad   = 0;
  last_time = 0;
  end_menu  = 0;

  psp_main_menu_init();

  psp_menu_dirty = 1;

  while (! end_menu)
  {
    psp_display_screen_menu();

    if (danzeff_mode) {
      danzeff_moveTo(-30, -30);
      danzeff_render();
    }
    psp_sdl_flip();

    while (1) {

      gp2xCtrlPeekBufferPositive(&c, 1);
      c.Buttons &= GP2X_ALL_BUTTON_MASK;
#ifdef LINUX_MODE
      usleep(60000);
#endif

      new_pad = c.Buttons;

      if ((old_pad != new_pad) || ((c.TimeStamp - last_time) > GP2X_MENU_MIN_TIME)) {
        last_time = c.TimeStamp;
        old_pad = new_pad;
        break;
      }
    }

    if (danzeff_mode) {

      danzeff_key = danzeff_readInput(c);
      if (danzeff_key > DANZEFF_START) {
        /* Disable utf8 */
        danzeff_key &= 0x7f;
        if (danzeff_key > ' ') {
          psp_main_menu_filename_add_char(danzeff_key);
        } else
        if (danzeff_key == DANZEFF_CLEAR) {
          psp_main_menu_filename_clear();
        } else
        if (danzeff_key == DANZEFF_DEL) {
          psp_main_menu_filename_del();
        }

      } else 
      if ((danzeff_key == DANZEFF_START ) || 
          (danzeff_key == DANZEFF_SELECT)) 
      {
        danzeff_mode = 0;
        old_pad = new_pad = 0;

        psp_kbd_wait_no_button();
      }
      continue;
    }

    if ((c.Buttons & (GP2X_CTRL_LTRIGGER|GP2X_CTRL_RTRIGGER|GP2X_CTRL_START)) ==
        (GP2X_CTRL_LTRIGGER|GP2X_CTRL_RTRIGGER|GP2X_CTRL_START)) {
      /* Exit ! */
      psp_sdl_exit(0);
    } else
    if ((new_pad & GP2X_CTRL_CROSS ) || 
        (new_pad & GP2X_CTRL_CIRCLE))
    {
      step = 0;

      if (new_pad & GP2X_CTRL_CROSS) {
        step = 1;
      } else
      if (new_pad & GP2X_CTRL_CIRCLE) {
        step = -1;
      }

      switch (cur_menu_id ) 
      {
        case MENU_PLAYER_COLOR : psp_main_menu_player_color( step );
        break;
        case MENU_SIDE_COLOR   : psp_main_menu_side_color( step );
        break;
        case MENU_DEPTH        : psp_main_menu_search_depth( step );
        break;
        case MENU_BOOK         : psp_main_menu_book( step );
        break;
        case MENU_SHOW_COORD   : psp_main_menu_show_coord( step );
        break;
        case MENU_LAST_MOVE    : psp_main_menu_last_move( step );
        break;
        case MENU_GAME_MODE    : psp_main_menu_game_mode( step );
        break;
# if 0 //LUDO:
        case MENU_TIME_LIMIT   : psp_main_menu_time_limit( step );
        break;
        case MENU_MOVE_LIMIT   : psp_main_menu_move_limit( step );
        break;
# endif
        case MENU_CLOCK        : psp_main_menu_clock( step );
        break;
        case MENU_SCREENSHOT   : psp_main_menu_screenshot();
                                 end_menu = 1;
        break;              
        case MENU_NEW_GAME     : psp_main_menu_new_game();
                                 end_menu = 1;
        break;                 
        case MENU_RESIGN_GAME  : psp_main_menu_resign_game();
                                 end_menu = 1;
        break;                 
        case MENU_LOAD_GAME    : if (psp_main_menu_load_game()) {
                                   end_menu = 1;
                                 }
        break;                 
        case MENU_SAVE_GAME    : psp_main_menu_save_game();
                                 end_menu = 1;
        break;                 
        case MENU_SAVE_CFG     : psp_main_menu_save_config();
        break;               
        case MENU_BACK         : end_menu = 1;
        break;                 
        case MENU_EXIT         : psp_main_menu_exit();
        break;                 
        case MENU_HELP         : psp_menu_dirty = 1;
                                 psp_help_menu();
                                 old_pad = new_pad = 0;
                                 psp_menu_dirty = 1;
        break;              
        case MENU_FILENAME     : if (step < 0) psp_main_menu_filename_del();
        break;
      }

    } else
    if(new_pad & GP2X_CTRL_START) {
      danzeff_mode = 1;
    } else
    if(new_pad & GP2X_CTRL_UP) {

      if (cur_menu_id > 0) cur_menu_id--;
      else                 cur_menu_id = MAX_MENU_ITEM-1;

    } else
    if(new_pad & GP2X_CTRL_DOWN) {

      if (cur_menu_id < (MAX_MENU_ITEM-1)) cur_menu_id++;
      else                                 cur_menu_id = 0;

    } else  
    if(new_pad & GP2X_CTRL_SQUARE) {
      /* Cancel */
      end_menu = -1;
    } else 
    if((new_pad & GP2X_CTRL_SELECT) == GP2X_CTRL_SELECT) {
      /* Back to game */
      end_menu = 1;
    }
  }

  if (end_menu > 0) {
    psp_main_menu_validate();
  }
 
  psp_kbd_wait_no_button();

  psp_sdl_clear_screen( GP2X_MENU_BLACK_COLOR );
  psp_sdl_flip();
  psp_sdl_clear_screen( GP2X_MENU_BLACK_COLOR );
  psp_sdl_flip();

  return 1;
}

